@@ -1,7 +1,7 @@
 \unset ECHO
 \i test/setup.sql
 
-SELECT plan(515);
+SELECT plan(542);
 --SELECT * FROM no_plan();
 
 -- This will be rolled back. :-)
@@ -324,7 +324,7 @@ SELECT * FROM check_test(
     false,
     'set_eq(prepared, select) fail extra',
     '',
-    '   Extra records:
+    '    Extra records:
         (44,Anna)'
 );
 
@@ -336,7 +336,7 @@ SELECT * FROM check_test(
     false,
     'set_eq(prepared, select) fail extras',
     '',
-    '   Extra records:
+    '    Extra records:
         [(](44,Anna|86,Angelina)[)]
         [(](44,Anna|86,Angelina)[)]',
     true
@@ -350,7 +350,7 @@ SELECT * FROM check_test(
     false,
     'set_eq(select, prepared) fail missing',
     '',
-    '   Missing records:
+    '    Missing records:
         (44,Anna)'
 );
 
@@ -362,7 +362,7 @@ SELECT * FROM check_test(
     false,
     'set_eq(select, prepared) fail missings',
     '',
-    '   Missing records:
+    '    Missing records:
         [(](44,Anna|86,Angelina)[)]
         [(](44,Anna|86,Angelina)[)]',
     true
@@ -376,7 +376,7 @@ SELECT * FROM check_test(
     false,
     'set_eq(select, select) fail extra & missing',
     '',
-    '   Extra records:
+    '    Extra records:
         (1,Jacob)
     Missing records:
         (44,Anna)'
@@ -390,7 +390,7 @@ SELECT * FROM check_test(
     false,
     'set_eq(select, select) fail extras & missings',
     '',
-    '   Extra records:
+    '    Extra records:
         [(](1,Jacob|87,Jackson)[)]
         [(](1,Jacob|87,Jackson)[)]
     Missing records:
@@ -407,7 +407,7 @@ SELECT * FROM check_test(
     false,
     'set_eq(values, values) fail mismatch',
     '',
-    '   Columns differ between queries:
+    '    Columns differ between queries:
         have: (integer,text)
         want: (text,integer)'
 );
@@ -421,7 +421,7 @@ SELECT * FROM check_test(
     false,
     'set_eq(values, values) fail column count',
     '',
-    '   Columns differ between queries:
+    '    Columns differ between queries:
         have: (integer)
         want: (text,integer)'
 );
@@ -446,7 +446,7 @@ SELECT * FROM check_test(
     false,
     'set_eq(sql, sql) fail type schema visibility',
     '',
-    '   Columns differ between queries:
+    '    Columns differ between queries:
         have: (text,__myfoo.text)
         want: (text,__myfoo.text,integer)'
 );
@@ -512,7 +512,7 @@ SELECT * FROM check_test(
     false,
     'bag_eq(prepared, select) fail extra',
     '',
-    '   Extra records:
+    '    Extra records:
         (44,Anna)'
 );
 
@@ -524,7 +524,7 @@ SELECT * FROM check_test(
     false,
     'bag_eq(prepared, select) fail extras',
     '',
-    '   Extra records:
+    '    Extra records:
         [(](44,Anna|86,Angelina)[)]
         [(](44,Anna|86,Angelina)[)]',
     true
@@ -538,7 +538,7 @@ SELECT * FROM check_test(
     false,
     'bag_eq(select, prepared) fail missing',
     '',
-    '   Missing records:
+    '    Missing records:
         (44,Anna)'
 );
 
@@ -550,7 +550,7 @@ SELECT * FROM check_test(
     false,
     'bag_eq(select, prepared) fail missings',
     '',
-    '   Missing records:
+    '    Missing records:
         [(](44,Anna|86,Angelina)[)]
         [(](44,Anna|86,Angelina)[)]',
     true
@@ -564,7 +564,7 @@ SELECT * FROM check_test(
     false,
     'bag_eq(select, select) fail extra & missing',
     '',
-    '   Extra records:
+    '    Extra records:
         (1,Jacob)
     Missing records:
         (44,Anna)'
@@ -578,7 +578,7 @@ SELECT * FROM check_test(
     false,
     'bag_eq(select, select) fail extras & missings',
     '',
-    '   Extra records:
+    '    Extra records:
         [(](1,Jacob|87,Jackson)[)]
         [(](1,Jacob|87,Jackson)[)]
     Missing records:
@@ -596,7 +596,7 @@ SELECT * FROM check_test(
     false,
     'bag_eq(values, values) fail mismatch',
     '',
-    '   Columns differ between queries:
+    '    Columns differ between queries:
         have: (integer,text)
         want: (text,integer)'
 );
@@ -610,7 +610,7 @@ SELECT * FROM check_test(
     false,
     'bag_eq(values, values) fail column count',
     '',
-    '   Columns differ between queries:
+    '    Columns differ between queries:
         have: (integer)
         want: (text,integer)'
 );
@@ -624,7 +624,7 @@ SELECT * FROM check_test(
     false,
     'bag_eq(values, values) fail missing dupe',
     '',
-    '   Extra records:
+    '    Extra records:
         (1,Anna)'
 );
 
@@ -671,7 +671,7 @@ SELECT * FROM check_test(
     false,
     'set_ne fail with column mismatch',
     '',
-    '   Columns differ between queries:
+    '    Columns differ between queries:
         have: (integer,text)
         want: (text,integer)'
 );
@@ -685,7 +685,7 @@ SELECT * FROM check_test(
     false,
     'set_ne fail with different col counts',
     '',
-    '   Columns differ between queries:
+    '    Columns differ between queries:
         have: (integer)
         want: (text,integer)'
 );
@@ -744,7 +744,7 @@ SELECT * FROM check_test(
     false,
     'bag_ne fail with column mismatch',
     '',
-    '   Columns differ between queries:
+    '    Columns differ between queries:
         have: (integer,text)
         want: (text,integer)'
 );
@@ -770,7 +770,7 @@ SELECT * FROM check_test(
     false,
     'bag_ne fail with column mismatch',
     '',
-    '   Columns differ between queries:
+    '    Columns differ between queries:
         have: (integer,text)
         want: (text,integer)'
 );
@@ -784,7 +784,7 @@ SELECT * FROM check_test(
     false,
     'bag_ne fail with different col counts',
     '',
-    '   Columns differ between queries:
+    '    Columns differ between queries:
         have: (integer)
         want: (text,integer)'
 );
@@ -886,7 +886,7 @@ SELECT * FROM check_test(
     false,
     'results_eq(nulls, nulls) fail',
     '',
-    '   Results differ beginning at row 2:
+    '    Results differ beginning at row 2:
         have: (,)
         want: NULL'
 );
@@ -900,7 +900,7 @@ SELECT * FROM check_test(
     false,
     'results_eq(prepared, select) fail',
     '',
-    '   Results differ beginning at row 3:
+    '    Results differ beginning at row 3:
         have: (44,Anna)
         want: (63,Angel)'
 );
@@ -914,7 +914,7 @@ SELECT * FROM check_test(
     false,
     'results_eq(select, prepared) fail missing last row',
     '',
-    '   Results differ beginning at row 7:
+    '    Results differ beginning at row 7:
         have: NULL
         want: (183,Antonio)'
 );
@@ -928,7 +928,7 @@ SELECT * FROM check_test(
     false,
     'results_eq(prepared, select) fail missing first row',
     '',
-    '   Results differ beginning at row 7:
+    '    Results differ beginning at row 7:
         have: (183,Antonio)
         want: NULL'
 );
@@ -949,7 +949,7 @@ SELECT * FROM check_test(
     false,
     'results_eq(values dupe, values)',
     '',
-    '   Results differ beginning at row 3:
+    '    Results differ beginning at row 3:
         have: (1,Anna)
         want: NULL'
 );
@@ -964,7 +964,7 @@ SELECT * FROM check_test(
     false,
     'results_eq(values null, values)',
     '',
-    '   Results differ beginning at row 1:
+    '    Results differ beginning at row 1:
         have: (1,)
         want: (1,Anna)'
 );
@@ -981,7 +981,7 @@ SELECT * FROM check_test(
     false,
     'results_eq(values, values) mismatch',
     '',
-    CASE WHEN pg_version_num() < 80400 THEN '   Results differ beginning at row 1:' ELSE '   Columns differ between queries:' END || '
+    CASE WHEN pg_version_num() < 80400 THEN '    Results differ beginning at row 1:' ELSE '    Number of columns or their types differ between the queries:' END || '
         have: (1,foo)
         want: (foo,1)'
 );
@@ -993,9 +993,23 @@ DECLARE
 BEGIN
     IF pg_version_num() < 80400 THEN
         -- 8.3 and earlier cast records to text, so subtlety is out.
-        RETURN NEXT pass('results_eq(values, values) subtle mismatch should fail');
-        RETURN NEXT pass('results_eq(values, values) subtle mismatch should have the proper description');
-        RETURN NEXT pass('results_eq(values, values) subtle mismatch should have the proper diagnostics');
+        -- Fake out pg_regress by running equivalent tests with fail().
+        FOR tap IN SELECT * FROM check_test(
+            fail('whatever'),
+            false,
+            'results_eq(values, values) subtle mismatch',
+            'whatever',
+            ''
+        ) AS a(b) LOOP RETURN NEXT tap.b; END LOOP;
+
+        FOR tap IN SELECT * FROM check_test(
+            fail('whatever'),
+            false,
+            'results_eq(values, values) integer type mismatch',
+            'whatever',
+            ''
+        ) AS a(b) LOOP RETURN NEXT tap.b; END LOOP;
+
     ELSE
         -- 8.4 does true record comparisions, yay!
         FOR tap IN SELECT * FROM check_test(
@@ -1006,9 +1020,19 @@ BEGIN
             false,
             'results_eq(values, values) subtle mismatch',
             '',
-            '   Columns differ between queries:
-        have: (1,foo)
-        want: (1,foo)' ) AS a(b) LOOP
+            '    Number of columns or their types differ between the queries' ) AS a(b) LOOP
+            RETURN NEXT tap.b;
+        END LOOP;
+
+        FOR tap IN SELECT * FROM check_test(
+            results_eq(
+                'VALUES (1::int), (2::int)',
+                'VALUES (1::bigint), (2::bigint)'
+            ),
+            false,
+            'results_eq(values, values) integer type mismatch',
+            '',
+            '    Number of columns or their types differ between the queries' ) AS a(b) LOOP
             RETURN NEXT tap.b;
         END LOOP;
     END IF;
@@ -1026,7 +1050,7 @@ SELECT * FROM check_test(
     false,
     'results_eq(values, values) fail column count',
     '',
-    CASE WHEN pg_version_num() < 80400 THEN '   Results differ beginning at row 1:' ELSE '   Columns differ between queries:' END || '
+    CASE WHEN pg_version_num() < 80400 THEN '    Results differ beginning at row 1:' ELSE '    Number of columns or their types differ between the queries:' END || '
         have: (1)
         want: (foo,1)'
 );
@@ -1161,7 +1185,7 @@ SELECT * FROM check_test(
     false,
     'set_has( missing1, expect )',
     '',
-    '   Missing records:
+    '    Missing records:
         (44,Anna)'
 );
 
@@ -1173,7 +1197,7 @@ SELECT * FROM check_test(
     false,
     'set_has(missing2, expect )',
     '',
-    '   Missing records:
+    '    Missing records:
         [(](44,Anna|86,Angelina)[)]
         [(](44,Anna|86,Angelina)[)]',
     true
@@ -1187,7 +1211,7 @@ SELECT * FROM check_test(
     false,
     'set_has((int,text), (text,int))',
     '',
-    '   Columns differ between queries:
+    '    Columns differ between queries:
         have: (integer,text)
         want: (text,integer)'
 );
@@ -1201,7 +1225,7 @@ SELECT * FROM check_test(
     false,
     'set_has((int), (text,int))',
     '',
-    '   Columns differ between queries:
+    '    Columns differ between queries:
         have: (integer)
         want: (text,integer)'
 );
@@ -1262,7 +1286,7 @@ SELECT * FROM check_test(
     false,
     'bag_has( prepared, dupes )',
     '',
-    '   Missing records:
+    '    Missing records:
         (44,Anna)'
 );
 
@@ -1285,7 +1309,7 @@ SELECT * FROM check_test(
     false,
     'bag_has( missing1, expect )',
     '',
-    '   Missing records:
+    '    Missing records:
         (44,Anna)'
 );
 
@@ -1297,7 +1321,7 @@ SELECT * FROM check_test(
     false,
     'bag_has(missing2, expect )',
     '',
-    '   Missing records:
+    '    Missing records:
         [(](44,Anna|86,Angelina)[)]
         [(](44,Anna|86,Angelina)[)]',
     true
@@ -1312,7 +1336,7 @@ SELECT * FROM check_test(
     false,
     'bag_has((int,text), (text,int))',
     '',
-    '   Columns differ between queries:
+    '    Columns differ between queries:
         have: (integer,text)
         want: (text,integer)'
 );
@@ -1326,7 +1350,7 @@ SELECT * FROM check_test(
     false,
     'bag_has((int), (text,int))',
     '',
-    '   Columns differ between queries:
+    '    Columns differ between queries:
         have: (integer)
         want: (text,integer)'
 );
@@ -1399,7 +1423,7 @@ SELECT * FROM check_test(
     false,
     'set_hasnt( prepared, value )',
     '',
-    '   Extra records:
+    '    Extra records:
         (44,Anna)'
 );
 
@@ -1408,7 +1432,7 @@ SELECT * FROM check_test(
     false,
     'set_hasnt( prepared, values )',
     '',
-    '   Extra records:
+    '    Extra records:
         [(](44,Anna|86,Angelina)[)]
         [(](44,Anna|86,Angelina)[)]',
     true
@@ -1423,7 +1447,7 @@ SELECT * FROM check_test(
     false,
     'set_hasnt((int,text), (text,int))',
     '',
-    '   Columns differ between queries:
+    '    Columns differ between queries:
         have: (integer,text)
         want: (text,integer)'
 );
@@ -1437,7 +1461,7 @@ SELECT * FROM check_test(
     false,
     'set_hasnt((int), (text,int))',
     '',
-    '   Columns differ between queries:
+    '    Columns differ between queries:
         have: (integer)
         want: (text,integer)'
 );
@@ -1495,7 +1519,7 @@ SELECT * FROM check_test(
     false,
     'bag_hasnt( prepared, value )',
     '',
-    '   Extra records:
+    '    Extra records:
         (44,Anna)'
 );
 
@@ -1504,7 +1528,7 @@ SELECT * FROM check_test(
     false,
     'bag_hasnt( prepared, values )',
     '',
-    '   Extra records:
+    '    Extra records:
         [(](44,Anna|86,Angelina)[)]
         [(](44,Anna|86,Angelina)[)]',
     true
@@ -1519,7 +1543,7 @@ SELECT * FROM check_test(
     false,
     'bag_hasnt((int,text), (text,int))',
     '',
-    '   Columns differ between queries:
+    '    Columns differ between queries:
         have: (integer,text)
         want: (text,integer)'
 );
@@ -1530,7 +1554,7 @@ SELECT * FROM check_test(
     false,
     'bag_hasnt((int), (text,int))',
     '',
-    '   Columns differ between queries:
+    '    Columns differ between queries:
         have: (integer)
         want: (text,integer)'
 );
@@ -1544,7 +1568,7 @@ SELECT * FROM check_test(
     false,
     'bag_hasnt( dupes, dupes )',
     '',
-    '   Extra records:
+    '    Extra records:
         (44,Anna)
         (44,Anna)'
 );
@@ -1558,7 +1582,7 @@ SELECT * FROM check_test(
     false,
     'bag_hasnt( value, dupes )',
     '',
-    '   Extra records:
+    '    Extra records:
         (44,Anna)'
 );
 
@@ -1607,7 +1631,7 @@ SELECT * FROM check_test(
     false,
     'set_eq(sql, array) extra record',
     '',
-    '   Extra records:
+    '    Extra records:
         (Anthony)'
 );
 
@@ -1620,7 +1644,7 @@ SELECT * FROM check_test(
     false,
     'set_eq(sql, array) missing record',
     '',
-    '   Missing records:
+    '    Missing records:
         (Alan)'
 );
 
@@ -1633,7 +1657,7 @@ SELECT * FROM check_test(
     false,
     'set_eq(sql, array) incompatible types',
     '',
-    '   Columns differ between queries:
+    '    Columns differ between queries:
         have: (text)
         want: (integer)'
 );
@@ -1647,7 +1671,7 @@ SELECT * FROM check_test(
     false,
     'set_eq(sql, array) incompatible types',
     '',
-    '   Columns differ between queries:
+    '    Columns differ between queries:
         have: (integer,text)
         want: (text)'
 );
@@ -1685,7 +1709,7 @@ SELECT * FROM check_test(
     false,
     'bag_eq(sql, dupe array) fail',
     '',
-    '   Missing records:
+    '    Missing records:
         (Anna)'
 );
 
@@ -1698,7 +1722,7 @@ SELECT * FROM check_test(
     false,
     'bag_eq(sql, array) extra record',
     '',
-    '   Extra records:
+    '    Extra records:
         (Anthony)'
 );
 
@@ -1711,7 +1735,7 @@ SELECT * FROM check_test(
     false,
     'bag_eq(sql, array) missing record',
     '',
-    '   Missing records:
+    '    Missing records:
         (Alan)'
 );
 
@@ -1724,7 +1748,7 @@ SELECT * FROM check_test(
     false,
     'bag_eq(sql, array) incompatible types',
     '',
-    '   Columns differ between queries:
+    '    Columns differ between queries:
         have: (text)
         want: (integer)'
 );
@@ -1738,7 +1762,7 @@ SELECT * FROM check_test(
     false,
     'bag_eq(sql, array) incompatible types',
     '',
-    '   Columns differ between queries:
+    '    Columns differ between queries:
         have: (integer,text)
         want: (text)'
 );
@@ -1800,7 +1824,7 @@ SELECT * FROM check_test(
     false,
     'set_ne(sql, array) incompatible types',
     '',
-    '   Columns differ between queries:
+    '    Columns differ between queries:
         have: (text)
         want: (integer)'
 );
@@ -1814,7 +1838,7 @@ SELECT * FROM check_test(
     false,
     'set_ne(sql, array) incompatible types',
     '',
-    '   Columns differ between queries:
+    '    Columns differ between queries:
         have: (integer,text)
         want: (text)'
 );
@@ -1876,7 +1900,7 @@ SELECT * FROM check_test(
     false,
     'bag_ne(sql, array) incompatible types',
     '',
-    '   Columns differ between queries:
+    '    Columns differ between queries:
         have: (text)
         want: (integer)'
 );
@@ -1890,7 +1914,7 @@ SELECT * FROM check_test(
     false,
     'bag_ne(sql, array) incompatible types',
     '',
-    '   Columns differ between queries:
+    '    Columns differ between queries:
         have: (integer,text)
         want: (text)'
 );
@@ -1954,7 +1978,7 @@ SELECT * FROM check_test(
     false,
     'results_eq(prepared, array) extra record',
     '',
-    '   Results differ beginning at row 7:
+    '    Results differ beginning at row 7:
         have: (Antonio)
         want: NULL'
 );
@@ -1967,7 +1991,7 @@ SELECT * FROM check_test(
     false,
     'results_eq(select, array) missing record',
     '',
-    '   Results differ beginning at row 5:
+    '    Results differ beginning at row 5:
         have: (Anthony)
         want: (Anna)'
 );
@@ -2162,7 +2186,7 @@ BEGIN
             false,
             'results_ne(values, values) mismatch',
             '',
-            '   Columns differ between queries:
+            '    Columns differ between queries:
         have: (1,foo)
         want: (foo,1)'
         ) AS a(b) LOOP
@@ -2178,7 +2202,7 @@ BEGIN
             false,
             'results_ne(values, values) subtle mismatch',
             '',
-            '   Columns differ between queries:
+            '    Columns differ between queries:
         have: (1,foo)
         want: (1,foo)' ) AS a(b) LOOP
             RETURN NEXT tap.b;
@@ -2190,7 +2214,7 @@ BEGIN
             false,
             'results_ne(values, values) fail column count',
             '',
-            '   Columns differ between queries:
+            '    Columns differ between queries:
         have: (1)
         want: (foo,1)'
         )  AS a(b) LOOP
@@ -2325,7 +2349,7 @@ BEGIN
             false,
             'is_empty(prepared) fail',
             '',
-            '   Unexpected records:
+            '    Unexpected records:
         (1,Jacob)
         (2,Emily)'
         ) AS a(b) LOOP
@@ -2337,6 +2361,73 @@ END;
 $$ LANGUAGE PLPGSQL;
 SELECT * FROM test_empty_fail();
 
+/****************************************************************************/
+-- Now test isnt_empty().
+SELECT * FROM check_test(
+    isnt_empty( 'SELECT 1', 'whatever' ),
+    true,
+    'isnt_empty(sql, desc)',
+    'whatever',
+    ''
+);
+
+SELECT * FROM check_test(
+    isnt_empty( 'SELECT 1 WHERE FALSE', 'whatever' ),
+    false,
+    'isnt_empty(sql, desc)',
+    'whatever',
+    ''
+);
+
+SELECT * FROM check_test(
+    isnt_empty( 'SELECT 1 WHERE FALSE' ),
+    false,
+    'isnt_empty(sql)',
+    '',
+    ''
+);
+
+SELECT * FROM check_test(
+    isnt_empty( 'SELECT 1' ),
+    true,
+    'isnt_empty(sql)',
+    '',
+    ''
+);
+
+PREPARE someset(boolean) AS SELECT * FROM names WHERE $1;
+SELECT * FROM check_test(
+    isnt_empty( 'EXECUTE someset(true)', 'whatever' ),
+    true,
+    'isnt_empty(prepared, desc)',
+    'whatever',
+    ''
+);
+
+SELECT * FROM check_test(
+    isnt_empty( 'EXECUTE someset(false)', 'whatever' ),
+    false,
+    'isnt_empty(prepared, desc)',
+    'whatever',
+    ''
+);
+
+SELECT * FROM check_test(
+    isnt_empty( 'EXECUTE someset(true)' ),
+    true,
+    'isnt_empty(prepared)',
+    '',
+    ''
+);
+
+SELECT * FROM check_test(
+    isnt_empty( 'EXECUTE someset(false)' ),
+    false,
+    'isnt_empty(prepared)',
+    '',
+    ''
+);
+
 /****************************************************************************/
 -- Test row_eq().
 PREPARE arow AS SELECT id, name FROM names WHERE name = 'Jacob';
@@ -2406,7 +2497,7 @@ BEGIN
             false,
             'row_eq(prepared, record, desc)',
             '',
-            '       have: (1,Jacob)
+            '        have: (1,Jacob)
         want: (1,Larry)'
         ) AS a(b) LOOP
             RETURN NEXT tap.b;
